//	---------------------------------------------------------------------------
//	$Header: /fp/fp1/67NSYY1QMG4igynohtna/anthony/debugstream.h,v 1.7 2002/06/01 15:25:32 ynohtna Exp $
//
//	(c) Copyright 2002 Anthony Bowyer-Lowe.  All rights reserved.
//
//	Anthony Bowyer-Lowe / ynohtna
//	ynohtna@ynohtna.org
//	http://www.ynohtna.org/
//	---------------------------------------------------------------------------
#if !defined(ALPHA_808_DEBUGSTREAM_H) && defined(WIN32)
#define ALPHA_808_DEBUGSTREAM_H

#include <ostream>
#include <sstream>
#include <string>

#define WIN32_LEAN_AND_MEAN
#define STRICT
#include <windows.h>


//	---------------------------------------------------------------------------
//	A stream that pipes its input upon a flush to the Windows debugging
//	console.
//	Very useful for leveraging custom class stream insertion operators into
//	debug code, and for using C++ stream semantics for debugging instead of
//	C-style DebugPrint(...) calls.
//
//	Usage:
//		anthony::debugstream dbg;
//		dbg << object << " - " << object.statistic() << std::endl;
//	or via an anonymous object
//		anthony::debugstream() << std::complex<float>(2.3f, -0.1f);
//
//	To disable debug output (in release builds for example), substitute
//	the debugstream (via a macro-controlled header inclusion) with either
//	an anthony::nullstream, a nilstream, an unattached anthony::teestream,
//	or even a std::ofilestream connected to "/dev/null" or "NUL".
//
//	Todo:
//	Implement synchronisation of debugstreams in multithreaded code through
//	a template policy parameter.
//	---------------------------------------------------------------------------
namespace anthony
{
#if 0
}	//	Nil indentum.
#endif


//	Custom streambuf class for the debugstream --------------------------------
template<typename E, typename T = std::char_traits<E> >
class basic_debugbuf : public std::basic_stringbuf<E, T>
{
public:
	basic_debugbuf() {	}
	~basic_debugbuf() {
		sync();
	}

protected:
	int sync() {
		output_debug_string(str().c_str());
		str(std::basic_string<E>());	//	Clear the string buffer.
		return 0;
	}
	void output_debug_string(const E* text) {
	}
};


//	Specialisations for char and wchar_t to call the correct Win32 APIs -------
template<>
void basic_debugbuf<char>::output_debug_string(const char* text) {
    ::OutputDebugStringA(text);
}

template<>
void basic_debugbuf<wchar_t>::output_debug_string(const wchar_t* text) {
    ::OutputDebugStringW(text);
}


//	The debugstream -----------------------------------------------------------
template<typename E, typename T = std::char_traits<E> >
class basic_debugstream : public std::basic_ostream<E, T>
{
	typedef basic_debugbuf<E, T>	debugbuf;

public:
	//	Connect the base ostream with the custom streambuf.
	basic_debugstream() : std::basic_ostream<E, T>(new debugbuf())
	{	}
	~basic_debugstream() {
		delete rdbuf();
	}
};


//	Typedefs for the obvious template instantions -----------------------------
typedef basic_debugstream<char>		debugstream;
typedef basic_debugstream<wchar_t>	debugstreamw;


//	Helper functor for debugging. Useful in conjunction with std::for_each ----
template<typename T>
struct debug_outputter
{
	void operator()(const T& t) {
		anthony::debugstream() << t << std::endl;
	}
};


//	---------------------------------------------------------------------------
}	//	namespace anthony


//	---------------------------------------------------------------------------
#endif	//	ALPHA_808_DEBUGSTREAM_H && WIN32
